# Pion!
class Pion
  # Attributes
  attr_accessor :options

  # Version number for Pion
  VERSION = "0.3.0"

  # Initialize a new Pion object
  def initialize(options)
    @options = method(:symbolize).call options

    fail ArgumentError, "No directory given!" unless options.key? :directory

    method(:set_options).call
  end

  # Generate urls from files
  def generate
    urls = []

    Dir.glob "#{options[:directory]}/**/*.#{options[:extension]}" do |file|
      hash = { file: file, url:  method(:get_url).call(file) }

      urls << hash unless file == "." || file == ".."
    end

    urls
  end

  private

  # Convert filename to clean url
  def get_url(file)
    url = file.gsub(/\A#{options[:directory]}/, "")
    url = url.gsub(/(\/index)?\.#{options[:extension]}\z/, "")
    url = url.gsub(/#{Regexp.escape(options[:separator])}/, "/")

    url == "" ? "/" : url
  end

  # Set directory or fail errors if it's faulty
  def set_directory
    dir = options[:directory]

    return if dir.is_a?(String) && File.directory?(dir)

    if !dir.is_a? String
      fail ArgumentError, "Directory must be a string."
    elsif !File.directory? dir
      fail ArgumentError, "Directory must be a valid directory."
    end
  end

  def set_default(type, default)
    options[type] = default unless options.key? type

    name = type.to_s.capitalize

    unless options[type].is_a? String
      fail ArgumentError, "#{name} must be a string"
    end
  end

  def set_options
    method(:set_directory).call

    method(:set_default).call :extension, "html"
    method(:set_default).call :separator, "/"
  end

  def symbolize(hash)
    fail ArgumentError, "No Hash given!" unless hash.is_a? Hash

    hash.reduce({}) do |memo, (key, value)|
      memo[key.to_sym] = value

      memo
    end
  end
end
